import { type Component, type JSX, Show } from "solid-js";
import { jwtDecode } from "jwt-decode";
import { Navigate } from "@solidjs/router";
import { saveToken } from "tauri-plugin-ios-shared-token-api";

export const isTokenValid = (): boolean => {
    const token = localStorage.getItem("access");

    if (token == null) {
        return false;
    }

    try {
        jwtDecode(token);
        return true;
    } catch (err) {
        return false;
    }
};

export const ProtectedRoute: Component<{ children?: JSX.Element }> = (
    props,
) => {
    const isValid = isTokenValid();

    if (isValid) {
        const token = localStorage.getItem("access");
        if (token == null) {
            throw new Error("unreachable");
        }

        saveToken(token)
            .then(() => console.log("Saved token!!!"))
            .catch((e) => console.error(e));
    }

    return (
        <Show when={isValid} fallback={<Navigate href="/login" />}>
            {props.children}
        </Show>
    );
};
